﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Microsoft.Win32;

namespace Data_Collector
{
    public partial class Data_Collector : Form
    {
        public Data_Collector()
        {
            InitializeComponent();

            for (int i = 0; i < EPTTransmitDevice.Length; ++i)
            {
                EPTTransmitDevice[i] = new Transfer();
            }

        }

        //Sting array to hold the Port names that match the VID = "403 PID = "6010"
        public string[] MatchingComPortList = new string[10];

        // Local Serial Port parameters From Properties Page
        public static string PortName = "COM4";
        public static string BaudRate = "115200";
        public static string DataBits = "8";
        public static string vParity = "None";
        public static string StopBits = "1";
        public static string pHandshake = "None";

        // Serial port send variables
        byte[] PutBuff;
        char[] PutChar;

        //Flag  for converting incoming stream to Hex
        public static bool HexConversion = false;
        public static int IdxHexChar = 0;
        public static bool hexConversionTabFlag = false;

        //Line Feed
        public static bool EchoTransmitCharacters = true;
        public static bool AppendLineFeed = false;

        //EPT Receive Command Parameters
        public const byte TRIGGER_IN_COMMAND = 0x08;
        public const byte TRANSFER_IN_COMMAND = 0x10;
        public const byte BLOCK_IN_COMMAND = 0x20;

        // EPT EndTerm parameters
        public const byte TRANSFER_ENDTERM_1 = 0x01;
        public const byte TRANSFER_ENDTERM_2 = 0x02;
        public const byte TRANSFER_ENDTERM_3 = 0x03;
        public const byte TRANSFER_ENDTERM_4 = 0x04;
        public const byte TRANSFER_ENDTERM_5 = 0x05;
        public const byte TRANSFER_ENDTERM_6 = 0x06;
        public const byte TRANSFER_ENDTERM_7 = 0x07;

        //Parameters for EPT Receive object Trigger
        public const byte TRIGGER_IN_BYTE_1 = 0x01;
        public const byte TRIGGER_IN_BYTE_2 = 0x02;
        public const byte TRIGGER_IN_BYTE_4 = 0x04;
        public const byte TRIGGER_IN_BYTE_8 = 0x08;
        public const byte TRIGGER_IN_BYTE_10 = 0x10;
        public const byte TRIGGER_IN_BYTE_20 = 0x20;
        public const byte TRIGGER_IN_BYTE_40 = 0x40;
        public const byte TRIGGER_IN_BYTE_80 = 0x80;

        //Parameters for EPT Receive object Transfer byte
        public const byte TRANSFER_IN_ADDRESS_1 = 0x1;
        public const byte TRANSFER_IN_ADDRESS_2 = 0x2;

        //Parameters for EPT Receive object Block
        public const byte BLOCK_IN_ADDRESS_1 = 0x1;
        public const byte BLOCK_IN_ADDRESS_2 = 0x2;

        //Create an array of the Transfer Class for device
        Transfer[] EPTTransmitDevice = new Transfer[8];

        // Main object loader
        private void Data_Collector_Load (object sender, System.EventArgs e)
        {

            //String buffer
            String PortText = "";

            //Index registers
            int Index = 0, EPTgroupNumber = 0;

            // Call the List Devices function
            List<string> names = ComPortNames("0403", "6010");

            // Get a list of serial port names.
            string[] ports;
            ports = SerialPort.GetPortNames();


            if (names.Count > 0)
            {
                foreach (String port in ports)
                {
                    //Compare port name with the found VID/PID 
                    //combinations. Add them to Matching port list
                    //and comboDevList
                    if (names.Contains(port))
                    {
                        MatchingComPortList[Index] = port;

                        if (Index == 0)
                        {
                            PortText = "EPT JTAG Blaster " + EPTgroupNumber;
                            Index++;
                        }
                        else
                        {
                            PortText = "EPT Serial Communications " + EPTgroupNumber++;
                            Index++;
                        }
                        cmbDevList.Items.Add(PortText);
                    }
                }
            }
            else
                MessageBox.Show("No EPT Devices found!");


        }


        private void SetButtonEnables()
        {
            btnOpenDevice.Enabled = false;
            btnCloseDevice.Enabled = true;
        }

        private void cmbDevList_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Get Port name from Combo Box
            PortName = MatchingComPortList[cmbDevList.SelectedIndex];
        }
        private void btnOpenDevice_Click(object sender, EventArgs e)
        {
            OpenSerialPort1();
            if (serialPort_AH.IsOpen)
            {
                lblDeviceConnected.Text = "Device Connected";
                SetButtonEnables();
            }

        }

        // Handle "Close" device button
        private void btnCloseDevice_Click(object sender, EventArgs e)
        {
            if (serialPort_AH.IsOpen)
            {
                CloseSerialPort1();
                lblDeviceConnected.Text = "Dis-Connected";
                //SetButtonEnables_Close();
            }
        }

        private void btnWriteByte_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, (byte)1);

            Thread.Sleep(5);

            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, (byte)1);
        }

        private void btnTransferReset_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, (byte)0);

            Thread.Sleep(5);

            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, (byte)0);
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            if (serialPort_AH.IsOpen)
            {
                CloseSerialPort1();
                lblDeviceConnected.Text = "Dis-Connected";
                //SetButtonEnables_Close();
            }
                btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

            lblDeviceConnected.Text = "";
            Application.Exit();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (serialPort_AH.IsOpen)
            {
                CloseSerialPort1();
                lblDeviceConnected.Text = "Dis-Connected";
                //SetButtonEnables_Close();
            }
                    btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

            lblDeviceConnected.Text = "";
            Application.Exit();
        }

        private void EPTParseReceive()
        {
            switch (EPTReceiveDevice.Command)
            {
                case TRANSFER_IN_COMMAND:
                    TransferOutReceive();
                    break;
                default:
                    break;
            }
        }


        public void TransferOutReceive()
        {
            string WriteRcvChar = "";
            WriteRcvChar = String.Format("{0}", (int)EPTReceiveDevice.Payload);
            tbDataBytes.AppendText(WriteRcvChar + " ");
        }

        private void btnResetBlock_Click(object sender, EventArgs e)
        {
            tbDataBytes.Clear();
        }

    }

    public class Transfer
    {
        public int Command;
        public int Address;
        public int Length;
        public int Payload;
        public byte[] cBlockBuf;
        public bool TransferPending;
        public uint Repititions;

        public Transfer()
        {
            Command = 0;
            Address = 0;
            Length = 0;
            Payload = 0;
            TransferPending = false;
            Repititions = 0;
        }

    }
	
}
