/*
  Copyright Earth People Technology Inc. 2016
  
  Data Collector Random Seed
  
  Platform: EPT-4CE6-AF-D2
    
*/
#define COMMANDDELAY	10	// ms to wait for the filling of Serial buffer

#define COMBUFFERSIZE	3	// Size of buffer for incoming numbers


  int startStopBit = 0;
  int count = 0;
  int ledPin = 13;     //Output
  int LEDExt = 15;     //Output
  int inPin8 = 11;     //Input
  int C_Enable = 17;   //Output
  
  int new_value = 0x00000000;
  int data_top_2bits = 0;
  int data_bottom_6bits = 0;
  int data_to_DPL = 0;
  
  int newP = 500;
  
  char commandBuffer[COMBUFFERSIZE+1];

// Fills the given buffer with bufferSize chars from a Serial object
void fillBuffer( \
	char *buffer, \
	byte bufferSize, \
	HardwareSerial::HardwareSerial* serial = &Serial );

void setup()
{
  
   int k ; 
   
   //Set pins D33-D41 as outputs
     for( k = 33 ; k <= 41 ; k++ )
      pinMode( k, OUTPUT ) ; // Sets Port C2-C9 to output pins
  
    // Set the output pins in the Output Write Enable Register
    REG_PIOC_OWER =  0x0000037E ; 
  
    // Disable writing to all other pins on the same port-
    REG_PIOC_OWDR = 0xFFFFFC81 ;  
    
    
  pinMode(C_Enable, OUTPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(LEDExt, OUTPUT);
  pinMode(inPin8, INPUT);
  
  randomSeed(analogRead(1));
  
  Serial.begin(115200);


}

void loop ()
{
  
   int i ;
   
   
   //Sample the Start/Stop switch
   //from the EPT-4CE6-AF   
   startStopBit = digitalRead(inPin8);
   
  delay(newP); //Delay in ms

   if(startStopBit) 
   {
     // Write a random number from 0 to 299
     //to the input of the EPT-4CE6-AF
      new_value = random(255);
      
      //Map the the transfer byte to pins available on
      //the DueProLogic
      data_top_2bits = (new_value & 0x000000c0)<<2;
      data_bottom_6bits = (new_value & 0x0000003F)<<1;
      
      //Or the re-mapped bits together and use the 
      //the ODSR register to transmit the bits
      data_to_DPL = data_top_2bits | data_bottom_6bits;
      REG_PIOC_ODSR = data_to_DPL;
         
      
      //Set the Write Enable Pin High
       digitalWrite(C_Enable, HIGH);
 
      //Set the LED Pin High
      digitalWrite(ledPin, HIGH);
      digitalWrite(LEDExt, HIGH);
      
      delay(newP); //Delay in ms

      //Set the LED Pin Low
      digitalWrite(ledPin, LOW);
      digitalWrite(LEDExt, LOW);
      
      //Set the Write Enable Pin Low
      digitalWrite(C_Enable, LOW);
      
   }
   
   	if ( Serial.available() > 0 ) {
		// Read the incoming byte
		char theChar = Serial.read();
			// Parse character
		switch (theChar) {
			case 's':	// 's' for setting time between samples
				// Wait for COMMANDDELAY ms to be sure that the Serial buffer is filled
				delay(COMMANDDELAY);

				fillBuffer( commandBuffer, COMBUFFERSIZE );

				// Convert buffer to integer
                                //commandBuffer = "32";
				newP = atoi( commandBuffer );

				// Display moving status indicator
				Serial.print("Setting the EPT prescaler to: ");
				Serial.println(newP);

				//Call set timing with newP
				//setIntervalTiming(newP);
				break;
			default:
				// Display error message
				Serial.print("ERROR: Command not found, it was: ");
				Serial.println(theChar);
				break;
                }
   }
  
}

//-----------------------------------------------------------------------------
// fillBuffer
//-----------------------------------------------------------------------------
// Fills the given buffer with bufferSize chars from a Serial object

void fillBuffer( char *buffer, byte bufferSize, HardwareSerial::HardwareSerial* serial )
{
	// Clean buffer
	memset( (void *)buffer, '\0', sizeof(char) * bufferSize );

	
	byte limit = ( bufferSize < serial->available() ) ? bufferSize : serial->available();


	// Fill buffer
	for ( byte i = 0; i < limit; i++ ) {

		buffer[i] = serial->read();
	}
}


