//#################################################################
//	Testbench for the 10M04 FPGA
//
//	File name:	tb_ept_10m04_top.v
//	Author:		Richard Jolly
//	Date:		February 20, 2018
//
//	Copyright 	Earth People Technology Inc. 2018
//
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			2/20/18 	1			Created			
//#                                 
//#
//#
//################################################################

`timescale 1ns/1ps

`include "../Testbench/tb_define.v"

module tb_ept_10m04_top;

   //----------------------------------------------
   // Parameter Declarations  
   //----------------------------------------------
	parameter   D_FLIP_FLOP              = 8'h13;
	parameter   OPERATORS                = 8'h14;


	//Shift Value for Operators
	parameter   SHIFT_OPERATOR           = 8'h03;

//---------------------------------------------------------------------------------------------------------------
//	wire and register declarations
//---------------------------------------------------------------------------------------------------------------
	reg                                  clk_50;
	reg                                  reset;
	
	//Registers for the D Flip Flop
	reg                         d;
	wire                        q;
	wire                        q_bar;
	
	//Registers for Operators
	reg  [7:0]                  addition_a;
	reg  [7:0]                  addition_b;
	wire [7:0]                  addition_result;
	
	reg  [7:0]                  subtraction_a;
	reg  [7:0]                  subtraction_b;
	wire [7:0]                  subtraction_result;
	
	reg  [7:0]                  multiplication_a;
	reg  [7:0]                  multiplication_b;
	wire [7:0]                  multiplication_result;
	
	reg  [7:0]                  division_a;
	reg  [7:0]                  division_b;
	wire [7:0]                  division_result;
	
	//Inputs and Outputs for Logical, Relational and Equality Operators
	
	reg  [7:0]                  logical_negation_a;
	reg  [7:0]                  logical_negation_b;
	wire [7:0]                  logical_negation_result;
	
	reg  [7:0]                  logical_and_a;
	reg  [7:0]                  logical_and_b;
	wire [7:0]                  logical_and_result;
	
	reg  [7:0]                  logical_or_a;
	reg  [7:0]                  logical_or_b;
	wire [7:0]                  logical_or_result;
	
	reg  [7:0]                  greater_lesser_than_a;
	reg  [7:0]                  greater_lesser_than_b;
	wire [7:0]                  greater_than_result;
	wire [7:0]                  lesser_than_result;
	
	reg  [7:0]                  greater_equal_than_a;
	reg  [7:0]                  greater_equal_than_b;
	wire [7:0]                  greater_equal_than_result;
	wire [7:0]                  lesser_equal_than_result;
	
	reg  [7:0]                  equality_a;
	reg  [7:0]                  equality_b;
	wire [7:0]                  equality_result;
	
	
	reg  [7:0]                  inequality_a;
	reg  [7:0]                  inequality_b;
	wire [7:0]                  inequality_result;
	
	//Inputs and Outputs for Reduction, Shift and Concatenation Operators
	
	reg  [7:0]                  bitwise_negation_a;
	reg  [7:0]                  bitwise_negation_b;
	wire [7:0]                  bitwise_negation_result;
	
	reg  [7:0]                  gate_a;
	reg  [7:0]                  gate_b;
	wire [7:0]                  nand_result;
	wire [7:0]                  or_result;
	wire [7:0]                  nor_result;
	wire [7:0]                  xor_result;
	wire [7:0]                  xnor_result;
	wire [7:0]                  right_shift_result;
	wire [7:0]                  left_shift_result;
	
	reg  [7:0]                  concatenation_a;
	reg  [7:0]                  concatenation_b;
	wire [7:0]                  concatenation_result;
	

//--------------------------------------------------------------------------------------------------------------
//	Generate Internal Clock with 50 MHz
//--------------------------------------------------------------------------------------------------------------

	initial 
	begin
		clk_50 = 0;
		forever
		//begin
			# `CYCLE_50 clk_50 = !clk_50;
		//end
	end


//-------------------------------------------------------------------------------------------------------------
//	Apply Stimulus
//-------------------------------------------------------------------------------------------------------------

	initial
	begin
	   reset                     = 1'b0;
	   addition_a                = 8'h0a;
	   addition_b                = 8'h0b;
	   subtraction_a             = 8'h0c;
	   subtraction_b             = 8'h0d;
	   multiplication_a          = 8'h0f;
	   multiplication_b          = 8'h10;
	   division_a                = 8'h12;
	   division_b                = 8'h13;
	   logical_negation_a        = 8'h15;
	   logical_negation_b        = 8'h16;
	   logical_and_a             = 8'h18;
	   logical_and_b             = 8'h19;
	   logical_or_a              = 8'h1b;
	   logical_or_b              = 8'h1c;
	   greater_lesser_than_a     = 8'h1e;
	   greater_lesser_than_b     = 8'h1f;
	   greater_equal_than_a      = 8'h22;
	   greater_equal_than_b      = 8'h23;
	   equality_a                = 8'h25;
	   equality_b                = 8'h27;
	   inequality_a              = 8'h29;
	   inequality_b              = 8'h2a;
       bitwise_negation_a        = 8'h2c;
       bitwise_negation_b        = 8'h2d;
       gate_a                    = 8'h30;
       gate_b                    = 8'h32;
       concatenation_a           = 8'h34;
       concatenation_b           = 8'h36;
	
	   
        //////////////////////////////////////////////////////
		//  Perform reset
        //////////////////////////////////////////////////////
		#(100 * `CYCLE)
	   reset                = 1'b1;
	   
	   
		#(100 * `CYCLE)
	   reset                = 1'b0;
	   
        //////////////////////////////////////////////////////
		//  Print the Title of the Section that is being tested
        //////////////////////////////////////////////////////
		
		call_title(OPERATORS);
		
		//Addition Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nAddition Operator: 0x%h + 0x%h = 0x%h\n",addition_a,addition_b,addition_result);

		//Subtraction Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nSubtraction Operator: 0x%h - 0x%h = 0x%h\n",subtraction_a,subtraction_b,subtraction_result);

		//Multiplication Operator
		#(100 * `CYCLE)
		$display("\n\n\n\Multiplication Operator: 0x%h * 0x%h = 0x%h\n",multiplication_a,multiplication_b,multiplication_result);

		//Division Operator
		#(100 * `CYCLE)
		if(division_b > division_a/2) division_b = division_a/2;
		#(100 * `CYCLE)
		$display("\n\n\n\Division Operator: 0x%h / 0x%h = 0x%h\n",division_a,division_b,division_result);

		//Logical Negation Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nLogical Negation Operator: !0x%h = 0x%h\n",logical_negation_a,logical_negation_result);

		//Logical AND Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nLogical AND Operator: 0x%h && 0x%h = 0x%h\n",logical_and_a,logical_and_b,logical_and_result);

		//Logical OR Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nLogical OR Operator: 0x%h || 0x%h = 0x%h\n",logical_or_a,logical_or_b,logical_or_result);

		//Greater Than Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nGreater Than Operator: Which number is greater? 0x%h or 0x%h: 0x%h is the greater number\n",greater_lesser_than_a,greater_lesser_than_b,greater_than_result);

		//Lesser Than Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nLesser Than Operator: Which number is lesser? 0x%h or 0x%h: 0x%h is the lesser number\n",greater_lesser_than_a,greater_lesser_than_b,lesser_than_result);

		//Greater Or Equal Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nGreater Or Equal Operator: Is 0x%h greater than or equal to 0x%h: %s\n",greater_equal_than_a,greater_equal_than_b,(greater_equal_than_result ? "No" : "Yes"));

		//Lesser Or Equal Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nLesser Or Equal Operator: Is 0x%h lesser than or equal to %s = %s\n",greater_equal_than_a,greater_equal_than_b,(lesser_equal_than_result ? "No" : "Yes"));

		//Equality Operator
		#(100 * `CYCLE)
		$display("\n\n\n\Equality Operator: Is 0x%h equal to 0x%h = %s\n",equality_a,equality_b,(equality_result ? "No" : "Yes"));

		//Inequality Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nInequality Operator: Is 0x%h not equal to 0x%h = %s\n",inequality_a,inequality_b,(inequality_result ? "No" : "Yes"));

		//Bitwise Negation Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nBitwise Negation Operator: 0x%h Bitwise Negated = 0x%h\n",bitwise_negation_a,bitwise_negation_result);

		//NAND Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nNAND Operator: 0x%h !& 0x%h = 0x%h\n",gate_a,gate_b,nand_result);

		//OR Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nOR Operator: 0x%h | 0x%h = 0x%h\n",gate_a,gate_b,or_result);

		//NOR Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nNOR Operator: 0x%h !| 0x%h = 0x%h\n",gate_a,gate_b,nor_result);

		//XOR Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nXOR Operator: 0x%h ^ 0x%h = 0x%h\n",gate_a,gate_b,xor_result);

		//XNOR Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nXNOR Operator: 0x%h ~^ 0x%h = 0x%h\n",gate_a,gate_b,xnor_result);

		//Right Shift Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nRight Shift Operator: 0x%h right shifed by 3 = 0x%h\n",gate_a,right_shift_result);

		//Left Shift Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nLeft Shift Operator: 0x%h left shifted by 3 = 0x%h\n",gate_a,left_shift_result);

		//Concatenation Operator
		#(100 * `CYCLE)
		$display("\n\n\n\nConcatenation Operator: Top four bits of 0x%h Concatenated with the bottom four bits of 0x%h = 0x%h\n",concatenation_a,concatenation_b,concatenation_result);

		// End of Simulation
		#(50000 * `CYCLE);
		stopsim;


	end	//stimulus
//--------------------------------------------------
//	Task: call title 
//--------------------------------------------------
   task call_title;
	input [7:0] select_title;
      begin
		case ( select_title ) 
		D_FLIP_FLOP:
		begin
		$display("\n\n\n\nSimulate the D Flip Flop\n");
		end
		OPERATORS:
		begin
		$display("\n\n\n\nSimulate the Verilog Operators\n");
		end
		default:
		$display("\n\n\n\nTest bench running....\n");
		endcase

      end
   endtask 

   //-----------------------------------------------
   // Task: stopsim
   //-----------------------------------------------
   task stopsim;
      begin
   	 $display("Simulation Finished at time %0t\n",$time);
	 $stop;
      end
   endtask // stopsim

  
   //-----------------------------------------------
   // Instantiate DUT
   //-----------------------------------------------
	
   EPT_10M04_AF_S2_Top          DUT
    (
	.ADDITION_A                (addition_a),
	.ADDITION_B                (addition_b),
	.ADDITION_RESULT           (addition_result),
	
	.SUBTRACTION_A             (subtraction_a),
	.SUBTRACTION_B             (subtraction_b),
	.SUBTRACTION_RESULT        (subtraction_result),
	
	.MULTIPLICATION_A          (multiplication_a),
	.MULTIPLICATION_B          (multiplication_b),
	.MULTIPLICATION_RESULT     (multiplication_result),
	
	.DIVISION_A                (division_a),
	.DIVISION_B                (division_b),
	.DIVISION_RESULT           (division_result),
	
	//Inputs and Outputs for Logical, Relational and Equality Operators
	
	.LOGICAL_NEGATION_A        (logical_negation_a),
	.LOGICAL_NEGATION_B        (logical_negation_b),
	.LOGICAL_NEGATION_RESULT   (logical_negation_result),
	
	.LOGICAL_AND_A             (logical_and_a),
	.LOGICAL_AND_B             (logical_and_b),
	.LOGICAL_AND_RESULT        (logical_and_result),
	
	.LOGICAL_OR_A              (logical_or_a),
	.LOGICAL_OR_B              (logical_or_b),
	.LOGICAL_OR_RESULT         (logical_or_result),
	
	.GREATER_LESSER_THAN_A     (greater_lesser_than_a),
	.GREATER_LESSER_THAN_B     (greater_lesser_than_b),
	.GREATER_THAN_RESULT       (greater_than_result),
	.LESSER_THAN_RESULT        (lesser_than_result),
	
	.GREATER_EQUAL_THAN_A      (greater_equal_than_a),
	.GREATER_EQUAL_THAN_B      (greater_equal_than_b),
	.GREATER_EQUAL_THAN_RESULT (greater_equal_than_result),
	.LESSER_EQUAL_THAN_RESULT  (lesser_equal_than_result),
	
	.EQUALITY_A                (equality_a),
	.EQUALITY_B                (equality_b),
	.EQUALITY_RESULT           (equality_result),
	
	
	.INEQUALITY_A              (inequality_a),
	.INEQUALITY_B              (inequality_b),
	.INEQUALITY_RESULT         (inequality_result),
	
	//Inputs and Outputs for Reduction, Shift and Concatenation Operators
	
	.BITWISE_NEGATION_A        (bitwise_negation_a),
	.BITWISE_NEGATION_B        (bitwise_negation_b),
	.BITWISE_NEGATION_RESULT   (bitwise_negation_result),
	
	.GATE_A                    (gate_a),
	.GATE_B                    (gate_b),
	.NAND_RESULT               (nand_result),
	.OR_RESULT                 (or_result),
	.NOR_RESULT                (nor_result),
	.XOR_RESULT                (xor_result),
	.XNOR_RESULT               (xnor_result),
	.SHIFT_OPERATOR            (SHIFT_OPERATOR),
	.RIGHT_SHIFT_RESULT        (right_shift_result),
	.LEFT_SHIFT_RESULT         (left_shift_result),
	
	.CONCATENATION_A           (concatenation_a),
	.CONCATENATION_B           (concatenation_b),
	.CONCATENATION_RESULT      (concatenation_result)
	
	);
	

endmodule // tb_ept_10m04_top.v


