//#################################################################
//	Testbench for the 10M04 FPGA
//
//	File name:	tb_ept_10m04_top.v
//	Author:		Richard Jolly
//	Date:		August 15, 2020
//
//	Copyright 	Earth People Technology Inc. 2020
//
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			8/15/20 	1			Created			
//#                                 
//#
//#
//################################################################

`timescale 1ns/1ps

`include "../Testbench/tb_define.v"

module tb_ept_10m04_top;

   //----------------------------------------------
   // Parameter Declarations  
   //----------------------------------------------
	parameter   D_FLIP_FLOP              = 8'h13;
	parameter   OPERATORS                = 8'h14;
	parameter   CONTROL_STATEMENTS       = 8'h15;

	//Shift Value for Operators
	parameter   SHIFT_OPERATOR           = 8'h03;
	
	//Control Statement
	parameter   IF_ELSE_COUNT_MAX        = 8'hf0;
	parameter   WHILE_COUNT_MAX          = 8'hf0;
	parameter   REPEAT_MAX_COUNT         = 8'h0a;

//---------------------------------------------------------------------------------------------------------------
//	wire and register declarations
//---------------------------------------------------------------------------------------------------------------
	reg                         clk_50;
	reg                         reset;

	//Registers for the D Flip Flop
	reg                         d;
	wire                        q;
	wire                        q_bar;
	
	//Registers for Operators
	reg  [7:0]                  addition_a;
	reg  [7:0]                  addition_b;
	wire [7:0]                  addition_result;
	
	reg  [7:0]                  subtraction_a;
	reg  [7:0]                  subtraction_b;
	wire [7:0]                  subtraction_result;
	
	reg  [7:0]                  multiplication_a;
	reg  [7:0]                  multiplication_b;
	wire [7:0]                  multiplication_result;
	
	reg  [7:0]                  division_a;
	reg  [7:0]                  division_b;
	wire [7:0]                  division_result;
	
	//Inputs and Outputs for Logical, Relational and Equality Operators
	
	reg  [7:0]                  logical_negation_a;
	reg  [7:0]                  logical_negation_b;
	wire [7:0]                  logical_negation_result;
	
	reg  [7:0]                  logical_and_a;
	reg  [7:0]                  logical_and_b;
	wire [7:0]                  logical_and_result;
	
	reg  [7:0]                  logical_or_a;
	reg  [7:0]                  logical_or_b;
	wire [7:0]                  logical_or_result;
	
	reg  [7:0]                  greater_lesser_than_a;
	reg  [7:0]                  greater_lesser_than_b;
	wire [7:0]                  greater_than_result;
	wire [7:0]                  lesser_than_result;
	
	reg  [7:0]                  greater_equal_than_a;
	reg  [7:0]                  greater_equal_than_b;
	wire [7:0]                  greater_equal_than_result;
	wire [7:0]                  lesser_equal_than_result;
	
	reg  [7:0]                  equality_a;
	reg  [7:0]                  equality_b;
	wire [7:0]                  equality_result;
	
	
	reg  [7:0]                  inequality_a;
	reg  [7:0]                  inequality_b;
	wire [7:0]                  inequality_result;
	
	//Inputs and Outputs for Reduction, Shift and Concatenation Operators
	
	reg  [7:0]                  bitwise_negation_a;
	reg  [7:0]                  bitwise_negation_b;
	wire [7:0]                  bitwise_negation_result;
	
	reg  [7:0]                  gate_a;
	reg  [7:0]                  gate_b;
	wire [7:0]                  nand_result;
	wire [7:0]                  or_result;
	wire [7:0]                  nor_result;
	wire [7:0]                  xor_result;
	wire [7:0]                  xnor_result;
	wire [7:0]                  right_shift_result;
	wire [7:0]                  left_shift_result;
	
	reg  [7:0]                  concatenation_a;
	reg  [7:0]                  concatenation_b;
	wire [7:0]                  concatenation_result;
	
	
	//Registers for Control Statements
	reg  [7:0]                  if_else_count;
	wire [7:0]                  if_else_result;
	reg  [7:0]                  case_input_register;
	wire [7:0]                  case_statement_result;
	reg  [7:0]                  while_counter;
	wire [7:0]                  while_result;
	reg  [7:0]                  for_loop_counter;
	wire [7:0]                  for_loop_result;
	reg  [7:0]                  repeat_loop_counter;
	wire [7:0]                  repeat_loop_result;
	
	reg  [7:0]                  index;
	reg  [7:0]                  loop_counter;
	
	

//--------------------------------------------------------------------------------------------------------------
//	Generate Internal Clock with 50 MHz
//--------------------------------------------------------------------------------------------------------------

	initial 
	begin
		clk_50 = 0;
		forever
		//begin
			# `CYCLE_50 clk_50 = !clk_50;
		//end
	end

//-------------------------------------------------------------------------------------------------------------
//	Apply Stimulus
//-------------------------------------------------------------------------------------------------------------

	initial
	begin
	   reset                     = 1'b1;
	   if_else_count             = 8'h00;
	   case_input_register       = 8'h00;
	   while_counter             = 8'h00;
	   for_loop_counter          = 8'h00;
       repeat_loop_counter	     = 8'h00;
	   loop_counter              = 8'h0a;
	   
        //////////////////////////////////////////////////////
		//  Perform reset
        //////////////////////////////////////////////////////
		#(100 * `CYCLE)
	   reset                = 1'b0;
	   
	   
		#(100 * `CYCLE)
	   reset                = 1'b1;
	   
        //////////////////////////////////////////////////////
		//  Print the Title of the Section that is being tested
        //////////////////////////////////////////////////////
		
		call_title(BLOCK_STATEMENTS);
		
		//If Else Statement
		#(100 * `CYCLE)
		$display("\n\n\n\nIf Else Statement executing now. Code will increment a counter");
		$display("\nuntil max count has been reached. An If statement compares each iteration ");
		$display("\nof the count to a MAX_COUNT value. Count Starts at 0x%h with Max Count = 0x%h",if_else_count,IF_ELSE_COUNT_MAX); 
				   
		while(if_else_result == 0) 
		begin
		   //$display("*");
		   #(100 * `CYCLE)
		   if_else_count = if_else_count + 1;
		end
		$display("\n\n\n\nIf Else Statement reached maximum count: 0x%h\n\n",if_else_result);
		
		
		//Case Statement
		#(100 * `CYCLE)
		$display("\n\n\n\nThe Case Statement compares a single input register to multiple");
		$display("\noutcome statements. The statement that matches the input register"); 
		$display("\nwill be selected. The results will be displayed. \n\n\n");
		repeat(10)
        begin
		    $display("Case Statement Input Register: 0x%h Output Statement: \n",
			         case_input_register,case_statement_result);
			#(100 * `CYCLE)
		    case_input_register = case_input_register + 1;
        end		

		//While Statement
		#(100 * `CYCLE)
		$display("\n\n\n\nThe While statement is not synthesizeable. So, the while ");
		$display("\nstatement will stay in the Testbench and a counter will be implemented");
		$display("\nin the User Code area. The Testbench sends an initial count value to the ");
		$display("\nuser code. The while statement will wait for the counter in the user code");
		$display("\nto expire. Then display the reults. The MAX COUNT is: 0x%h\n",WHILE_COUNT_MAX);
		while_counter = 1;
		while(while_result == 0) 
		begin
		   $display("*");
		   #(100 * `CYCLE);
		end
		$display("\n\n\n\nWhile Statement reached maximum count: 0x%h\n\n",while_result);

		//For Loop Statement
		#(100 * `CYCLE)
		$display("\n\n\n\The For Loop allows repeated statements to execute within");
        $display("\na pre-set range. This means the for loop will start a count value");
		$display("\nand increment until it reaches a max count value. Each increment");
		$display("\nwill cause the statement to execute.\n");
		for_loop_counter = 10;
		#(100 * `CYCLE)
		$display("\nFor Loop Count started at: 0x%h", for_loop_counter);
		$display("\nThe MAX COUNT is 0x%h", for_loop_counter);
		$display("\nFor Loop result is 0x%h", for_loop_result);
		//Repeat Statement
		#(100 * `CYCLE)
		$display("\n\n\n\The Repeat Loop is not synthesizeable. So, it will implemented");
		$display("\nin a counter in the user code and use the repeat keyword in the testbench.");
		$display("\nStart the counter by changing the initial value to non-zero. Use repeat");
		$display("\nwith a variable that decrements on each simulation cycle. The repeat will");
		$display("\ncomplete when the varaible reaches zero.\n");
		#(100 * `CYCLE);
		repeat(10)
		begin
		     repeat_loop_counter = 2;
			 #(100 * `CYCLE);
		     loop_counter = REPEAT_MAX_COUNT - repeat_loop_result;
			 repeat_loop_counter = 1;
			 #(100 * `CYCLE);
		     $display("\nRepeat Loop Counter is 0x%h", loop_counter);
		end


		// End of Simulation
		#(50000 * `CYCLE);
		stopsim;


	end	//stimulus
//--------------------------------------------------
//	Task: call title 
//--------------------------------------------------
   task call_title;
	input [7:0] select_title;
      begin
		case ( select_title ) 
		D_FLIP_FLOP:
		begin
		$display("\n\n\n\nSimulate the D Flip Flop\n");
		end
		OPERATORS:
		begin
		$display("\n\n\n\nSimulate the Verilog Operators\n");
		end
		CONTROL_STATEMENTS:
		begin
		$display("\n\n\n\nSimulate the Control Statements Code\n");
		end
		BLOCK_STATEMENTS:
		begin
		$display("\n\n\n\nSimulate the Block Statements Code\n");
		end
		default:
		$display("\n\n\n\nTest bench running....\n");
		endcase

      end
   endtask 


   //-----------------------------------------------
   // Task: stopsim
   //-----------------------------------------------
   task stopsim;
      begin
   	 $display("Simulation Finished at time %0t\n",$time);
	 $stop;
	 //$close(logfile);
      end
   endtask // stopsim

  
   //-----------------------------------------------
   // Instantiate DUT
   //-----------------------------------------------
	
   EPT_10M04_AF_S2_Top          DUT
    (
	.CLK                        (clk_50),
	.RST_N                      (reset),
	
	.IF_ELSE_COUNTER_1          (if_else_count),
	.IF_ELSE_RESULT_1           (if_else_result),
	
	.CASE_COUNTER_2             (case_input_register),
	.CASE_RESULT_2              (case_statement_result),
	
	.WHILE_COUNTER_3            (while_counter),
	.WHILE_RESULT_3             (while_result),
	
	.FOR_LOOP_COUNTER_4         (for_loop_counter),
	.FOR_LOOP_RESULT_4          (for_loop_result),

	.REPEAT_LOOP_COUNTER_5      (repeat_loop_counter),
	.REPEAT_LOOP_RESULT_5       (repeat_loop_result)
	);
	

endmodule // tb_ept_10m04_top.v


