//#################################################################
//	Testbench for the 10M04 FPGA
//
//	File name:	tb_ept_10m04_top.v
//	Author:		Richard Jolly
//	Date:		August 15, 2020
//
//	Copyright 	Earth People Technology Inc. 2020
//
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			8/15/20 	1			Created			
//#                                 
//#
//#
//################################################################

`timescale 1ns/1ps

`include "../Testbench/tb_define.v"

module tb_ept_10m04_top;

   //----------------------------------------------
   // Parameter Declarations  
   //----------------------------------------------
	parameter   D_FLIP_FLOP              = 8'h13;
	parameter   OPERATORS                = 8'h14;
	parameter   CONTROL_STATEMENTS       = 8'h15;
	parameter   BLOCK_STATEMENTS         = 8'h16;

	//Shift Value for Operators
	parameter   SHIFT_OPERATOR           = 8'h03;
	
	//Control Statement
	parameter   IF_ELSE_COUNT_MAX        = 8'hf0;
	parameter   WHILE_COUNT_MAX          = 8'hf0;
	parameter   REPEAT_MAX_COUNT         = 8'h0a;

//---------------------------------------------------------------------------------------------------------------
//	wire and register declarations
//---------------------------------------------------------------------------------------------------------------
	reg                        clk_50;
	reg                        reset;

	//Registers for the D Flip Flop
	reg                         d;
	wire                        q;
	wire                        q_bar;
	
	//Registers for Operators
	reg  [7:0]                  addition_a;
	reg  [7:0]                  addition_b;
	wire [7:0]                  addition_result;
	
	reg  [7:0]                  subtraction_a;
	reg  [7:0]                  subtraction_b;
	wire [7:0]                  subtraction_result;
	
	reg  [7:0]                  multiplication_a;
	reg  [7:0]                  multiplication_b;
	wire [7:0]                  multiplication_result;
	
	reg  [7:0]                  division_a;
	reg  [7:0]                  division_b;
	wire [7:0]                  division_result;
	
	//Inputs and Outputs for Logical, Relational and Equality Operators
	
	reg  [7:0]                  logical_negation_a;
	reg  [7:0]                  logical_negation_b;
	wire [7:0]                  logical_negation_result;
	
	reg  [7:0]                  logical_and_a;
	reg  [7:0]                  logical_and_b;
	wire [7:0]                  logical_and_result;
	
	reg  [7:0]                  logical_or_a;
	reg  [7:0]                  logical_or_b;
	wire [7:0]                  logical_or_result;
	
	reg  [7:0]                  greater_lesser_than_a;
	reg  [7:0]                  greater_lesser_than_b;
	wire [7:0]                  greater_than_result;
	wire [7:0]                  lesser_than_result;
	
	reg  [7:0]                  greater_equal_than_a;
	reg  [7:0]                  greater_equal_than_b;
	wire [7:0]                  greater_equal_than_result;
	wire [7:0]                  lesser_equal_than_result;
	
	reg  [7:0]                  equality_a;
	reg  [7:0]                  equality_b;
	wire [7:0]                  equality_result;
	
	
	reg  [7:0]                  inequality_a;
	reg  [7:0]                  inequality_b;
	wire [7:0]                  inequality_result;
	
	//Inputs and Outputs for Reduction, Shift and Concatenation Operators
	
	reg  [7:0]                  bitwise_negation_a;
	reg  [7:0]                  bitwise_negation_b;
	wire [7:0]                  bitwise_negation_result;
	
	reg  [7:0]                  gate_a;
	reg  [7:0]                  gate_b;
	wire [7:0]                  nand_result;
	wire [7:0]                  or_result;
	wire [7:0]                  nor_result;
	wire [7:0]                  xor_result;
	wire [7:0]                  xnor_result;
	wire [7:0]                  right_shift_result;
	wire [7:0]                  left_shift_result;
	
	reg  [7:0]                  concatenation_a;
	reg  [7:0]                  concatenation_b;
	wire [7:0]                  concatenation_result;
	
	
	//Registers for Control Statements
	reg  [7:0]                  if_else_count;
	wire [7:0]                  if_else_result;
	reg  [7:0]                  case_input_register;
	wire [7:0]                  case_statement_result;
	reg  [7:0]                  while_counter;
	wire [7:0]                  while_result;
	reg  [7:0]                  for_loop_counter;
	wire [7:0]                  for_loop_result;
	reg  [7:0]                  repeat_loop_counter;
	wire [7:0]                  repeat_loop_result;
	
	reg  [7:0]                  index;
	reg  [7:0]                  loop_counter;
	
	//Registers for the Block Statments
	reg                         select;
	reg                         in_a;
	reg                         in_b;
	wire                        y_comb;
	wire                        y_sync;
	reg                         enable;
	wire                        out_1;
	reg                         in_1;
	wire                        out_2;
	reg  [31:0]                 in_3;
	wire                        out_3;
	reg                         select_3;
	wire [31:0]                 compare_4;
	reg  [31:0]                 data_a_4;
	reg  [31:0]                 data_b_4;
	
	

//--------------------------------------------------------------------------------------------------------------
//	Generate Internal Clock with 50 MHz
//--------------------------------------------------------------------------------------------------------------

	initial 
	begin
		clk_50 = 0;
		forever
			# `CYCLE_50 clk_50 = !clk_50;
	end

//--------------------------------------------------------------------------------------------------------------
//	Generate the select signal for the always block
//--------------------------------------------------------------------------------------------------------------

	initial 
	begin
		select = 0;
		forever
			# `SELECT_CYCLE_100 select = !select;
	end
	
//--------------------------------------------------------------------------------------------------------------
//	Inital Block
//--------------------------------------------------------------------------------------------------------------
			
    initial 
    begin
       in_a = 1;
       in_b = 0;
    end



//-------------------------------------------------------------------------------------------------------------
//	Apply Stimulus
//-------------------------------------------------------------------------------------------------------------

	initial
	begin
	   reset                     = 1'b1;
	   if_else_count             = 8'h00;
	   case_input_register       = 8'h00;
	   while_counter             = 8'h00;
	   for_loop_counter          = 8'h00;
       repeat_loop_counter	     = 8'h00;
	   loop_counter              = 8'h0a;
	   in_1                      = 1'b0;
	   enable                    = 1'b0;
	   data_a_4                  = 32'h0;
       data_b_4                  = 32'h0;
	
	   
        //////////////////////////////////////////////////////
		//  Perform reset
        //////////////////////////////////////////////////////
		#(100 * `CYCLE)
	   reset                = 1'b0;
	   
	   
		#(100 * `CYCLE)
	   reset                = 1'b1;
	   
       //Delay from reset
       #(5000 * `CYCLE)

 
 
        //////////////////////////////////////////////////////
		//  Print the Title of the Section that is being tested
        //////////////////////////////////////////////////////
		
		call_title(BLOCK_STATEMENTS);
		
		
		//Assignment Testbench
		//Tri-State Buffer results
       #(5000 * `CYCLE) //Delay for Simulation
       in_1 = 1'b1;
	   #(100 * `CYCLE)//Delay for enable
       $display("The assignment result is: 0x%b\n\n",out_1);

       #(5000 * `CYCLE) //Delay for Simulation
       in_1 = 1'b0;
	   #(100 * `CYCLE)//Delay for enable
       $display("The assignment result is: 0x%b\n\n",out_1);


	   //Assignment Testbench
	   //Buffer results
       #(5000 * `CYCLE) //Delay for Simulation
       enable = 1'b1;
	   #(100 * `CYCLE)//Delay for in_2
       $display("The assignment result is: 0x%b\n\n",out_2);

       #(5000 * `CYCLE) //Delay for Simulation
       enable = 1'b0;
	   #(100 * `CYCLE)//Delay for in_2
       $display("The assignment result is: 0x%b\n\n",out_2);

	   //Functions Testbench
	   //Function: Calculate Parity, Call results
       #(5000 * `CYCLE) //Delay for Simulation
	   in_3 = `FUNCTIONS_PARITY_DATA;
	   #(100 * `CYCLE)//Delay for in_3
       select_3 = 1'b1;
	   #(100 * `CYCLE)//Delay for select_3
       $display("The assignment result is: 0x%b\n\n",out_3);

	   //Tasks Testbench
	   //Task: Compare to words, Call results
       #(5000 * `CYCLE) //Delay for Simulation
	   data_a_4 = `TASK_COMPARE_DATA_A;
	   data_b_4 = `TASK_COMPARE_DATA_B;
	   #(100 * `CYCLE)//Delay for compare data word A and word B
	   if(compare_4 == 32'hffffffff)
          $display("The data word A and data word B are the same.\n\n");
	   else
	      $display("The data word A and data word B are different.\n\n");

	   
		// End of Simulation
		#(50000 * `CYCLE);
		stopsim;


	end	//stimulus
//--------------------------------------------------
//	Task: call title 
//--------------------------------------------------
   task call_title;
	input [7:0] select_title;
      begin
		case ( select_title ) 
		D_FLIP_FLOP:
		begin
		$display("\n\n\n\nSimulate the D Flip Flop\n");
		end
		OPERATORS:
		begin
		$display("\n\n\n\nSimulate the Verilog Operators\n");
		end
		CONTROL_STATEMENTS:
		begin
		$display("\n\n\n\nSimulate the Control Statements Code\n");
		end
		BLOCK_STATEMENTS:
		begin
		$display("\n\n\n\nSimulate the Block Statements Code\n");
		end
		default:
		$display("\n\n\n\nTest bench running....\n");
		endcase

      end
   endtask 


   //-----------------------------------------------
   // Task: stopsim
   //-----------------------------------------------
   task stopsim;
      begin
   	 $display("Simulation Finished at time %0t\n",$time);
	 $stop;
	 //$close(logfile);
      end
   endtask // stopsim

   //-----------------------------------------------
   // Instantiate DUT
   //-----------------------------------------------
	
   EPT_10M04_AF_S2_Top          DUT
    (
	.CLK                        (clk_50),
	.RST_N                      (reset),
	
	.SELECT                     (select),
	
	.IN_A                       (in_a),	
	.IN_B                       (in_b),
	
	.Y_COMB                     (y_comb),
	.Y_SYNC                     (y_sync),
	
	.OUT_1                      (out_1),
	.IN_1                       (in_1),

	.ENABLE                     (enable),
	.OUT_2                      (out_2),

	.OUT_3                      (out_3),
	.IN_3                       (in_3),
	.SELECT_3                   (select_3),

	.OUT_4                      (compare_4),
	.IN_A_4                     (data_a_4),
	.IN_B_4                     (data_b_4)
	
	);
	
  

endmodule // tb_ept_10m04_top.v


