//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2020
//#
//#
//# File Name:  EPT_10M04_AF_S2_Top.v
//# Author:     Earth People Technology
//# Date:       August 15, 2020
//# Revision:   A
//#
//# Development: EPT Verilog Controls Statements 
//# Application: Altera MAX10 FPGA
//# Description: This file contains verilog code which access
//#              the I/O's of the FPGA. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			8/15/20 	1			Created
//#
//#									
//#
//#######################################################################

`timescale 1ns/1ps

//************************************************************************
//* Module Declaration
//************************************************************************

// Block Statements Code
    module EPT_10M04_AF_S2_Top 
	( 
	
	//Inputs and Outputs for Verilog Block Statements
	
	input  wire                CLK,
	input  wire                RST_N,
		

	input wire                 SELECT,       

	input wire                 IN_A,       
	input wire                 IN_B, 
	
	output reg                 Y_COMB,         
	output reg                 Y_SYNC,

	output wire                OUT_1,
	input wire                 IN_1,	

	input wire                 ENABLE,	
	output wire                OUT_2,
	
	output wire                OUT_3,
	input wire  [31:0]         IN_3,
    input wire                 SELECT_3,	
	
	output reg [31:0]          OUT_4,
	input wire  [31:0]         IN_A_4,
	input wire  [31:0]         IN_B_4
	
	);

	
   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   
   
   
//***************************************************************************
//*     Internal Signals and Registers Declarations
//***************************************************************************
   reg y_nosense;
   wire data_1;
   wire data_2;
   reg  parity;
   
//***************************************************************************
//* 	Internal Signal Assignments	
//***************************************************************************
   assign data_2 = 1;
	
   assign OUT_3 = parity;

//***************************************************************************
//* 	Always Combinational Block
//***************************************************************************
   always@(IN_A or IN_B or SELECT) 
   begin
      if(SELECT == 0)
         Y_COMB = IN_A;
      else
         Y_COMB = IN_B;
   end
	   
//***************************************************************************
//* 	Always Synchronous Block
//***************************************************************************
   always@(posedge CLK) 
   begin
      if(!RST_N)
      begin
         Y_SYNC <= 0;
      end 
      else 
      begin
         if(SELECT == 0)
            Y_SYNC <= IN_A;
         else
            Y_SYNC <= IN_B;
      end
   end

//***************************************************************************
//* 	Always Block, No Sensitivity List
//***************************************************************************
   initial
   begin
      y_nosense = 0;
   end

   always 
   begin
     #100 y_nosense = !y_nosense;
   end
	   
//***************************************************************************
//* 	Assignments
//***************************************************************************
   assign OUT_1 = IN_1;
   
   assign OUT_2 = ENABLE ? data_2 : 1'bz;

//***************************************************************************
//* 	Functions -- Listing
//***************************************************************************
 function parity_3;
    input [31:0] data_3;
    integer i;
    begin
      parity_3 = 0;
      for (i= 0; i < 32; i = i + 1) begin
        parity_3 = parity_3 ^ data_3[i];
      end
    end
    endfunction

//***************************************************************************
//* 	Functions -- Calling
//***************************************************************************
		always@(posedge CLK) 
		begin
           if(!RST_N)
           begin
			  parity <= 0;
           end 
           else 
	       begin
		      if(SELECT_3 == 1)
		        parity <= parity_3(IN_3);
		   end
		end


//***************************************************************************
//* 	Tasks -- Listing
//***************************************************************************
 task compare_4;
    input  [31:0] data_a_4;
    input  [31:0] data_b_4;
	output [31:0] result_4;
    integer i;
    begin
	  result_4 = 0;
      for (i= 0; i < 32; i = i + 1) 
	  begin
        if(data_a_4[i] == data_b_4[i])
		  result_4[i] = 1;
        else
          result_4[i] = 0;		
      end
    end
    endtask

//***************************************************************************
//* 	Tasks -- Calling
//***************************************************************************
		always@(posedge CLK) 
		begin
           if(!RST_N)
           begin
			  OUT_4 <= 0;
           end 
           else 
	       begin
		      compare_4(IN_A_4, IN_B_4, OUT_4);
		   end
		end

  endmodule
