module	ft_245_state_macine
	(	
	CLK,
	RST_N,

	
	USB_RXF_N,
	USB_TXE_N,
//	USB_DATA,

	USB_RD_N,
	USB_WR,
	USB_TEST,
	
	USB_REGISTER_DECODE,
	USB_DATA_IN,
	USB_DATA_OUT,
	
	DATA_BYTE_READY,
	RSB_INT_EN,
	
	WRITE_EN,
	WRITE_READY,
	WRITE_BYTE,
	WRITE_COMPLETE,
	
	STATE_OUT,
	TEST_SIGNAL1,
	TEST_SIGNAL2
	
	);

   //----------------------------------------------
   // Parameter Declarations  
   //----------------------------------------------
	parameter 	IDLE							= 0,
				ASSERT_RD_N	 					= 1,
				WAIT_FOR_RD_COMPLETION			= 2,
				WAIT_FOR_DE_ASSERT_RD_N			= 3,
				DE_ASSERT_RD_N					= 4,
				ASSERT_WR						= 5,
				WAIT_FOR_WR_COMPLETION			= 6,
				DE_ASSERT_WR					= 7;

   //--------------------------------------------------
   // IO Port Declarations                           
   //--------------------------------------------------
 
	input							CLK;
	input							RST_N;

	
	input							USB_RXF_N;
	input							USB_TXE_N;
//	inout	[7:0]					USB_DATA;

	output							USB_RD_N;
	output							USB_WR;
	output							USB_TEST;
	
	output	[7:0]					USB_REGISTER_DECODE;
	input	[7:0]					USB_DATA_IN;
	output	[7:0]					USB_DATA_OUT;
	
	output							DATA_BYTE_READY;
	input							RSB_INT_EN;
	
	input							WRITE_EN;
	output							WRITE_READY;
	input	[7:0]					WRITE_BYTE;
	output							WRITE_COMPLETE;
	
	output	[3:0]					STATE_OUT;
	output							TEST_SIGNAL1;
	output							TEST_SIGNAL2;

  //------------------------------------------------
   // Wire and Register Declarations                
   //------------------------------------------------
	reg 	[8:0] 					state, next;

	reg								USB_RD_N;
	reg								USB_WR;
	wire								USB_TEST;
	
//	wire	[7:0]						USB_DATA;
	
	wire	[7:0]					USB_REGISTER_DECODE;
	wire	[7:0]					USB_DATA_OUT;
	reg								DATA_BYTE_READY;
	
	reg								WRITE_READY;
	reg								WRITE_COMPLETE;



	
	//Read/Write Control
	reg							read_complete;
	reg							read_complete_reg;
	reg	[7:0]					read_complete_cntr;
	reg							write_complete;
	reg							write_complete_reg;
	reg	[7:0]					write_complete_cntr;
	
	//Write control registers
	reg							write_en_meta_1;
	reg							write_en_meta_2;
	reg							write_en_reg;

	
	//Internal Data buses
	reg	[7:0]					data_out_bus;
	reg	[7:0]					read_cycle_data_bus;
	wire	[7:0]					data_in;
	
	//Register the FT245 read control inputs 
	reg							usb_rxf_n_reg;
	reg							usb_rxf_n_meta_1;
	reg							usb_rxf_n_meta_2;

	//Register the FT245 write control inputs 
	reg							usb_txe_n_reg;
	reg							usb_txe_n_meta_1;
	reg							usb_txe_n_meta_2;


	//Address Control
	
   //TEST ONLY
	reg	[3:0]						STATE_OUT;
	reg	[3:0]						STATE_IN;
	wire							TEST_SIGNAL1;
	wire							TEST_SIGNAL2;
	
	

`ifdef SIM
   reg [8*26:1] state_name;
`endif

   //--------------------------------------------------
   // Signal Assignments
   //--------------------------------------------------   

//	assign				USB_DATA = ( state[WAIT_FOR_WR_COMPLETION] ) ? 8'haa : 8'hzz;	
	assign				USB_REGISTER_DECODE = read_cycle_data_bus;
	assign				data_in = 8'h55;
	assign				USB_DATA_OUT = ( state[WAIT_FOR_WR_COMPLETION] ) ? WRITE_BYTE : 8'hzz;
	assign				TEST_SIGNAL1 = USB_RXF_N;
	assign				TEST_SIGNAL2 = usb_rxf_n_reg;
	assign				USB_TEST = 1'b0;
	
   //-----------------------------------------------
   // Create a State Test Output Signal
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  STATE_OUT <= 0;
	else
	  begin
	     if (state[IDLE])
            STATE_OUT <= 4'h0;
	     if (state[ASSERT_RD_N])
	       STATE_OUT <= 4'h1;
	     if (state[WAIT_FOR_RD_COMPLETION])
	       STATE_OUT <= 4'h2;
	     if (state[WAIT_FOR_DE_ASSERT_RD_N])
	       STATE_OUT <= 4'h3;
	     if (state[DE_ASSERT_RD_N])
	       STATE_OUT <= 4'h4;
	     if (state[ASSERT_WR])
	       STATE_OUT <= 4'h5;
	     if (state[WAIT_FOR_WR_COMPLETION])
	       STATE_OUT <= 4'h6;
	     if (state[DE_ASSERT_WR])
	       STATE_OUT <= 4'h7;
	  end // else: !if(!RST_N)
     end // always @ (posedge CLK or negedge RST_N)


`ifdef SIM
   always @(posedge CLK)
   begin
	   if ( state == ( 1 << IDLE ))
		   state_name = "IDLE";
	   else if ( state == ( 1 << ASSERT_RD_N ))
		   state_name = "ASSERT_RD_N";
	   else if ( state == ( 1 << WAIT_FOR_RD_COMPLETION ))
		   state_name = "WAIT_FOR_RD_COMPLETION";
	   else if ( state == ( 1 << WAIT_FOR_DE_ASSERT_RD_N ))
		   state_name = "WAIT_FOR_DE_ASSERT_RD_N";
	   else if ( state == ( 1 << DE_ASSERT_RD_N ))
		   state_name = "DE_ASSERT_RD_N";
	   else if ( state == ( 1 << ASSERT_WR ))
		   state_name = "ASSERT_WR";
	   else if ( state == ( 1 << WAIT_FOR_WR_COMPLETION ))
		   state_name = "WAIT_FOR_WR_COMPLETION";
	   else if ( state == ( 1 << DE_ASSERT_WR ))
		   state_name = "DE_ASSERT_WR";
	end
`endif



   //-----------------------------------------------
   // Create WRITE_COMPLETE for the write cycle
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
		WRITE_COMPLETE <= 0;
	end
	else
	  begin
		if( state[DE_ASSERT_WR] )
		begin
			WRITE_COMPLETE <= 1;
		end
		else if( state[IDLE] )
		begin
			WRITE_COMPLETE <= 0;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)




   //-----------------------------------------------
   // Create WRITE_READY for the write cycle
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
		WRITE_READY <= 0;
	end
	else
	  begin
		if( state[WAIT_FOR_WR_COMPLETION] )
		begin
			WRITE_READY <= 1;
		end
		else if( state[DE_ASSERT_WR] )
		begin
			WRITE_READY <= 0;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Register the USB_RXF_N input
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
		usb_rxf_n_reg <= 1;
		usb_rxf_n_meta_1 <= 0;
		usb_rxf_n_meta_2 <= 0;
	end
	else
	  begin
		if(  state[WAIT_FOR_DE_ASSERT_RD_N] )
		begin
			usb_rxf_n_meta_1 <= 0;
			usb_rxf_n_meta_2 <= 0;
			usb_rxf_n_reg <= 1;
		end
		else if( !USB_RXF_N && !usb_rxf_n_meta_1 && !usb_rxf_n_meta_2 )
		begin
			usb_rxf_n_meta_1 <= 1;
			usb_rxf_n_meta_2 <= 0;
			usb_rxf_n_reg <= 1;
		end
		else if( USB_RXF_N && usb_rxf_n_meta_1 && !usb_rxf_n_meta_2 )
		begin
			usb_rxf_n_meta_1 <= 1;
			usb_rxf_n_meta_2 <= 1;
			usb_rxf_n_reg <= 1;
		end
		else if( !USB_RXF_N && usb_rxf_n_meta_1 && usb_rxf_n_meta_2 )
		begin
			usb_rxf_n_meta_1 <= 1;
			usb_rxf_n_meta_2 <= 1;
			usb_rxf_n_reg <= 0;
		end
//		else if( USB_RXF_N && ( state[WAIT_FOR_DE_ASSERT_RD_N] ))
		else if(  state[WAIT_FOR_DE_ASSERT_RD_N] )
		begin
			usb_rxf_n_meta_1 <= 0;
			usb_rxf_n_meta_2 <= 0;
			usb_rxf_n_reg <= 1;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Register the WRITE_EN input
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
			write_en_meta_1 <= 0;
			write_en_meta_2 <= 0;
			write_en_reg <= 1;
	end
	else
	  begin
		if( state[DE_ASSERT_WR] )
		begin
			write_en_meta_1 <= 0;
			write_en_meta_2 <= 0;
			write_en_reg <= 1;
		end
		else if( WRITE_EN && !write_en_meta_1 && !write_en_meta_2 )
		begin
			write_en_meta_1 <= 1;
			write_en_meta_2 <= 0;
			write_en_reg <= 1;
		end
		else if( WRITE_EN && write_en_meta_1 && !write_en_meta_2 )
		begin
			write_en_meta_1 <= 1;
			write_en_meta_2 <= 1;
			write_en_reg <= 0;
		end
		else if( !WRITE_EN && write_en_meta_1 && write_en_meta_2 )
		begin
			write_en_meta_1 <= 1;
			write_en_meta_2 <= 1;
			write_en_reg <= 1;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)


   //-----------------------------------------------
   // write_complete register will initiate the FT 245
   // to enter the write cycle mode. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
					write_complete_reg <= 0;
					write_complete_cntr <= 0;
					write_complete <= 0;
					usb_txe_n_reg <= 0;
					USB_WR <= 0;
	end
	else
	  begin
		if( state[ASSERT_WR] && !write_complete_reg )
		begin
			write_complete_reg <= 1;
			write_complete <= 0;
		end
		else if( state[WAIT_FOR_WR_COMPLETION] && write_complete_reg )
		begin
			USB_WR <= 1;
//			write_complete_reg <= 1;
			write_complete_cntr <= write_complete_cntr + 1;
			write_complete <= 0;
				if ( write_complete_cntr >= 8'h0f )
				begin
					USB_WR <= 0;	
					write_complete_reg <= 0;
					write_complete_cntr <= 0;
					write_complete <= 0;
						
				end
		end
		else if( state[WAIT_FOR_WR_COMPLETION] && !write_complete_reg )
		begin
				if( USB_TXE_N && !usb_txe_n_reg )
				begin
						usb_txe_n_reg <= 1;
				end
				else if ( !USB_TXE_N && usb_txe_n_reg )
				begin
					write_complete <= 1;
				end
		end
		else if( state[DE_ASSERT_WR] )
		begin
					write_complete_reg <= 0;
					write_complete_cntr <= 0;
					write_complete <= 0;
					usb_txe_n_reg <= 0;
					USB_WR <= 0;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)



   //-----------------------------------------------
   // Create read_complete read
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
		read_complete <= 0;
	end
	else
	  begin
		if( state[ASSERT_RD_N] && !read_complete_reg )
		begin
			read_complete_reg <= 1;
			read_complete <= 0;
		end
		else if( state[WAIT_FOR_RD_COMPLETION] && read_complete_reg )
		begin
			read_complete_reg <= 1;
			read_complete_cntr <= read_complete_cntr + 1;
			
				if ( read_complete_cntr <= 8'h0f )
				begin	
					read_complete_reg <= 0;
					read_complete_cntr <= 0;
					read_complete <= 1;
						
				end
		end
		else if( state[IDLE] )
		begin
					read_complete_reg <= 0;
					read_complete_cntr <= 0;
					read_complete <= 0;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Create DATA_BYTE_READY read
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
		DATA_BYTE_READY <= 0;
	end
	else
	  begin
		if( state[WAIT_FOR_RD_COMPLETION] )
		begin
			DATA_BYTE_READY <= 1;
		end
		else if( RSB_INT_EN )
		begin
			DATA_BYTE_READY <= 0;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Create USB_DATA read cycle
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
		read_cycle_data_bus <= 0;
	end
	else
	  begin
		if( state[WAIT_FOR_RD_COMPLETION] )
		begin
			read_cycle_data_bus <= USB_DATA_IN;
		end
		else if( state[DE_ASSERT_RD_N] )
		begin
			read_cycle_data_bus <= read_cycle_data_bus;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Create USB_DATA write cycle
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
		data_out_bus <= 0;
	end
	else
	  begin
		if( state[ASSERT_WR] )
		begin
			data_out_bus <= data_in;
		end
		else if( state[DE_ASSERT_WR] )
		begin
			data_out_bus <= data_out_bus;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)




   //-----------------------------------------------
   // Create USB_RD_N output
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
		USB_RD_N <= 1;
	end
	else
	  begin
		if( state[ASSERT_RD_N] )
		begin
			USB_RD_N <= 0;
		end
		else if( state[DE_ASSERT_RD_N] )
		begin
			USB_RD_N <= 1;
		end
	   end

     end // always @ (posedge CLK or negedge RST_N)

 


   

   //-----------------------------------------------
   // Finite  State Machine
   //-----------------------------------------------
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 9'h000;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end



     // State Definitions
   always @ ( state or usb_rxf_n_reg  or usb_txe_n_reg or read_complete or
		write_complete or write_en_reg or WRITE_EN ) 
     begin
	next = 9'h000;

	if (state[IDLE])
	  begin
	     if (  !usb_rxf_n_reg )
	       next[ASSERT_RD_N] = 1'b1;
	     else if (  !write_en_reg )
	       next[ASSERT_WR] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end
	
	if (state[ASSERT_RD_N])
	begin
			next[WAIT_FOR_RD_COMPLETION] = 1'b1;
	end

	if (state[WAIT_FOR_RD_COMPLETION])
	begin
		if ( read_complete )
			next[WAIT_FOR_DE_ASSERT_RD_N] = 1'b1;
		else 
			next[WAIT_FOR_RD_COMPLETION] = 1'b1;
	end

	if (state[WAIT_FOR_DE_ASSERT_RD_N])
	begin
		if ( usb_rxf_n_reg )
			next[DE_ASSERT_RD_N] = 1'b1;
		else 
			next[WAIT_FOR_DE_ASSERT_RD_N] = 1'b1;
	end

	if (state[DE_ASSERT_RD_N])
			next[IDLE] = 1'b1;
	
	if (state[ASSERT_WR])
	begin
			next[WAIT_FOR_WR_COMPLETION] = 1'b1;
	end

	if (state[WAIT_FOR_WR_COMPLETION])
	begin
		if ( write_complete )
			next[DE_ASSERT_WR] = 1'b1;		  
		else
			next[WAIT_FOR_WR_COMPLETION] = 1'b1;
	end


	if (state[DE_ASSERT_WR])
	begin
			next[IDLE] = 1'b1;
	end
	


	end//end of state machine

	
	
   
endmodule 